#!/usr/bin/env python

""" cexbot - command_utils.py

Default command line utitlities to run cexbot

@TODO improve the long helps
"""

import os
import sys
import logging
import argparse

import cexbot.config
import cexbot.tasks

from cexbot.db import DbManager
from cexbot.cexapi import CexAPI
from cexbot.updater import check_update
from cexbot.timer import ReqTimer
from cexbot.cex import CexMethods


def main(argv=[]):
  args = get_parser()

  verbose = 1
  if args.verbose:
    verbose = 2
  if args.debug:
    verbose = 3

  if verbose>2:
    log_level=logging.DEBUG
  elif verbose==2:
    log_level=logging.INFO
  elif verbose==1:
    log_level=logging.WARNING
  elif verbose<1:
    log_level=logging.ERROR

  logging.basicConfig(level=log_level, format="%(asctime)s %(levelname)s: %(message)s")

  # make sure this is always above command parsing
  cexbot.config.first_run()

  if verbose == 3:
    print args

  if args.command == 'config':
    if args.list:
      return cexbot.config.list()
    elif args.edit:
      return cexbot.config.edit_config()
    elif args.testauth:
      return cexbot.config.test_auth()
    elif args.name and args.value:
      v = cexbot.config.set(args.name, args.value)
      return cexbot.config.cprint(args.name)
    elif args.name:
      return cexbot.config.cprint(args.name)
    logging.error('Invalid config option')
    return 1

  elif args.command == 'update':
    return check_update()

  if args.task == 'cleardata':
    return cexbot.config.clear_userdata()


  config = cexbot.config.get_config()
  ac = CexAPI(config.get('cex', 'username'), config.get('cex', 'apikey'), config.get('cex', 'secret'))
  dbi = DbManager()
  cx = CexMethods(ac, dbi)

  if args.task == 'getbalance':
    logging.info("Balance: %s" % ac.get_balance())
    return True

  if args.task == 'initdb':
    return dbi.initdb()

  if args.task == 'getmarket':
    return ac.get_market()

  if args.task == 'getprice':
    return ac.get_market_quote()

  if args.task == 'order':
    amount = args.amount
    price = args.price
    r = ac.place_order(amount, price)
    logging.info("Ordered: %s" % r)

  if args.task == 'updatequotes':
    logging.info('Running updatequotes')
    ticker_timer = ReqTimer(2, cx.update_ticker)
    ticker_timer.start()

  if args.task == 'buybalance':
    logging.info('Running buybalance')
    balance_timer = ReqTimer(5, ac.buy_balance)
    balance_timer.start()

  # @TODO __import__
  # if args.task in cexbot.tasks:
    # cexbot.tasks[args.task]()

def get_parser():
  parser = argparse.ArgumentParser(prog='cexbot-cli', description='cexbot')

  parser.add_argument('-v', dest='verbose', action='store_true', help='verbose output')
  parser.add_argument('--version', dest='version', action='store_true', help='show version')
  parser.add_argument('-d', dest='debug', action='store_true', help='debug output')

  subparsers = parser.add_subparsers(description='available subcommands', dest="command")

  parser_config = subparsers.add_parser('config', help='config options')
  parser_config.add_argument('--list', dest='list', action='store_true', help='list configuration variables')
  parser_config.add_argument('--edit', dest='edit', action='store_true', help='edit configuration directly')
  parser_config.add_argument('--testauth', dest='testauth', action='store_true', help='test authentication credentials')
  parser_config.add_argument('name', type=str, help='option name', nargs='?')
  parser_config.add_argument('value', type=str, help='option value', nargs='?')

  parser_task = subparsers.add_parser('task', help='modify tasks')
  parser_task.add_argument('name', type=str, help='task name')

  parser_order = subparsers.add_parser('order', help='order')
  parser_order.add_argument('-p', dest='price', action='store_true', help='price')
  parser_order.add_argument('-a', dest='amount', action='store_true', help='amount')

  parser_update = subparsers.add_parser('update', help='check for updates')

  return parser.parse_args()

def cl_error(msg=""):
  print >> sys.stderr, msg

def run_cl(argv=[]):
  try:
    raise SystemExit(main(sys.argv))
  except KeyboardInterrupt:
    cl_error('Interrupted.')
    raise SystemExit(-1)

def run_gui(argv=[]):
  print "GUI coming soon."
  return None
  try:
    import cexbot.gui
    cexbot.gui.main()
  except Exception, e:
    print "Error: %s" % str(e)




#!/usr/bin/env python

"""cexbot
"""
# Contributions from Django

# Copyright (c) Django Software Foundation and individual contributors.
# All rights reserved.

import sys
import os

VERSION = (0, 0, 13, 'alpha', 1)

__clsname__ = 'cexbot'
__author__ = 'Nik Cubrilovic <nikcub@gmail.com>'
__email__ = 'nikcub@gmail.com'
__url__ = 'http://nikcub.github.com/cexbot'
__license__ = 'BSD'
__copyright__ = 'Copyright (c) 2013, Nik Cubrilovic. All rights reserved.'


def get_version(version=None, semantic=False):
  if version is None:
    version = VERSION
  assert version[3] in ('alpha', 'beta', 'rc', 'final')
  parts = 2 if version[2] == 0 else 3
  main = '.'.join(str(x) for x in version[:parts])
  sub = ''
  if version[3] == 'alpha' and version[4] == 0:
    git_changeset = get_git_changeset()
    if git_changeset:
      sub = '.dev%s' % git_changeset
  elif version[3] != 'final':
    mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
    sub = mapping[version[3]] + str(version[4])
  if not semantic:
    return main + sub
  else:
    return main + '-' + sub

def get_status(version=None):
  if version is None:
    version = VERSION
  assert version[3] in ('alpha', 'beta', 'rc', 'final')
  return version[3]

def write_version(file_path=None, semantic=False):
  if not file_path:
    if not '__file__' in globals():
      cwd = os.path.abspath(os.getcwd())
    else:
      cwd = os.path.dirname(os.path.abspath(__file__))
    file_path = os.path.realpath(os.path.join(cwd, '..'))
  version_file = os.path.join(file_path, 'VERSION')
  print file_path
  with open(version_file, 'w') as f:
    f.write(get_version(semantic=semantic))

def get_git_changeset():  # pragma: nocover
  """Returns a numeric identifier of the latest git changeset.

  The result is the UTC timestamp of the changeset in YYYYMMDDHHMMSS format.
  This value isn't guaranteed to be unique, but collisions are very
  unlikely, so it's sufficient for generating the development version
  numbers.
  """
  repo_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
  git_log = subprocess.Popen('git log --pretty=format:%ct --quiet -1 HEAD',
                           stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                           shell=True, cwd=repo_dir,
                           universal_newlines=True)
  timestamp = git_log.communicate()[0]
  try:
    timestamp = datetime.datetime.utcfromtimestamp(int(timestamp))
  except ValueError:  # pragma: nocover
    return None   # pragma: nocover
  return timestamp.strftime('%Y%m%d%H%M%S')


if __name__ == '__main__':
  from cexbot.command_utils import run_cl
  sys.exit(run_cl())




#!/usr/bin/env python
"""
  cexbot.db - database abstraction
"""

import logging
import sqlite3 as lite

import config

class DbManager(object):

  def __init__(self, path_db=None):
    if not path_db:
      path_db = config.get_db_path()
    self.conn = lite.connect(path_db)
    self.conn.execute('pragma foreign_keys = on')
    self.conn.commit()
    self.cur = self.conn.cursor()

  def __del__(self):
    self.conn.close()

  def query(self, arg):
    self.cur.execute(arg)
    self.conn.commit()
    return self.cur

  def check_table(self, name):
    self.cur.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='%s'" % (name))


  def getdb(self):
    con = lite.connect(DB_NAME)
    return con

  def init(self):
    self.cur.execute("CREATE TABLE IF NOT EXISTS quotes (time INTEGER primary key, last integer, volume integer, high integer, low integer, bid integer, ask integer)")
    logging.info('Database Initialized')
    return True


#!/usr/bin/env python
"""
	cexbot.cexapi
"""


class CexMethods(object):

	def __init__(self, ap, db):
		self.ap = ap
		self.db = db
		pass

	def update_ticker():
		r = self.ap.req('ticker')
		con = self.db.getdb()
		cur = con.cursor()
		sql = "INSERT INTO quotes VALUES(%s, %s, %s, %s, %s, %s, %s)" % (r['timestamp'], r['last'], r['volume'], r['high'], r['low'], r['bid'], r['ask'])
		try:
			cur.execute(sql)
		except Exception, e:
			pass
		con.commit()






#!/usr/bin/env python
"""
  cexbot.cexapi
"""

import requests
import requests.exceptions
import hmac
import hashlib
import json
import logging
from time import time

class CexAPI(object):

  CEX_API_BASE = 'https://cex.io/api'

  # format is (URI, private, required params)
  api_methods = {
    'ticker': ('/ticker/GHS/BTC', False, None),
    'book': ('/order_book/GHS/BTC', False, None),
    'history': ('/trade_history/GHS/BTC', False, None),
    'balance': ('/balance/', True),
    'orders': ('/open_orders/GHS/BTC', True),
    'cancel_order': ('/cancel_order/', True, ('id')),
    'place_order': ('/place_order/GHS/BTC', True, ('type', 'amount', 'price')),
  }

  access_token = {'username': None, 'apikey': None, 'secret': None}

  headers = {
    'User-Agent' : 'bot-cex.io-',
    }

  def __init__(self, username, apikey, secret):
    if not username or not apikey or not secret:
      logging.error("Need username, apikey and secret - please edit your config")
      return False
    self.access_token['username'] = username
    self.access_token['apikey'] = apikey
    self.access_token['secret'] = secret

  def get_params(self):
    nonce = int(time())
    message = str(nonce) + self.access_token['username'] + self.access_token['apikey']
    sig = hmac.new(self.access_token['secret'], msg=message, digestmod=hashlib.sha256).hexdigest().upper()
    return {'key': self.access_token['apikey'], 'signature': sig, 'nonce': nonce}

  def get_headers(self):
    headers = self.headers
    # @TODO add headers
    return headers

  def req(self, meth, extras={}):
    if not meth in self.api_methods:
      logging.error("Error: No method %s" % (meth))
      return False
    req_method = self.api_methods[meth]
    req_uri = self.CEX_API_BASE + req_method[0]
    try:
      if req_method[1]:
        params = self.get_params()
        params.update(extras)
        r = requests.post(req_uri, headers=self.headers, data=params)
      else:
        r = requests.post(req_uri, headers=self.headers)
    except requests.exceptions.ConnectionError, e:
      logging.error("Error: connection")
      return false
    if r.headers['Content-Type'] != 'text/json' or not r.text:
      logging.error("Request error (%s)" % r.text[:200])
      return False
    try:
      c = json.loads(r.text)
      if 'error' in c:
        logging.error("API: %s" % c['error'])
        return False
    except ValueError:
      logging.error("Content loading error")
      return False
    return c

  def get_balance(self):
    try:
      br = self.req('balance')
      if br:
        return br['BTC']['available']
    except KeyError:
      return False

  def buy_market(self, amount):
    pass

  def place_order(self, amount, price, typ='buy'):
    extras = {
      'type': typ,
      'amount': amount,
      'price': price
    }
    r = self.req('place_order', extras)
    if 'id' in r:
      return r['id']
    logging.error("Order error")
    return False

  def buy_balance(self, balance_threshold=0.0001):
    balance = self.get_balance()
    if not balance:
      return False
    if balance < balance_threshold:
      return False
    price = self.get_market_quote()
    if not price:
      logging.error('price error')
      return False
    price = float(price)
    balance = float(balance)
    amount = balance / price
    order_total = amount * price
    if amount > 0.0001:
      logging.info("Buy %s at %s for total balance %s (of %s)" % (amount, price, order_total, balance))
      e = self.place_order(amount, price)
      logging.info("Order Id: %s" % e)

  def get_market(self):
    try:
      quotes = self.req('book')
      if quotes:
        for q in quotes[:5]:
          print q
    except Exception:
      return False

  def get_market_quote(self):
    # @todo check the quantity here
    try:
      ask = self.req('book')
      if ask:
        return ask['asks'][0][0]
    except Exception:
      return False




#!/usr/bin

import threading

class ReqTimer:
	def __init__(self, tempo, target, args= [], kwargs={}):
		self._target = target
		self._args = args
		self._kwargs = kwargs
		self._tempo = tempo

	def _run(self):
		self._timer = threading.Timer(self._tempo, self._run)
		self._timer.start()
		self._target(*self._args, **self._kwargs)

	def start(self):
		self._timer = threading.Timer(self._tempo, self._run)
		self._timer.start()

	def stop(self):
		self._timer.cancel()





#!/usr/bin/env python
"""
	cexbot.config - config abstraction
"""

import os
import logging
import ConfigParser
import subprocess

from cexbot.appdirs import AppDirs
from cexbot.db import DbManager
from cexbot.cexapi import CexAPI

ad = AppDirs("cexbot", "cexbot")
_parser = ConfigParser.SafeConfigParser(allow_no_value=True)

DB_NAME = 'tradedata.db'
CNF_NAME = 'cex.cnf'
CONFIG_DEFAULTS = {
	'cex': {
		'username': 'username',
		'apikey': 'api',
		'secret': 'secret',
		}
	}
CONFIG_REQUIRED = {
	'cex': ('username', 'apikey', 'secret'),
}

class BadConfig(Exception):
	pass

def first_run():
	path_config = get_conf_path()
	path_db = get_db_path()
	if not os.path.isdir(ad.user_data_dir):
		os.mkdir(ad.user_data_dir)
	if not os.path.isfile(path_config):
		logging.debug("Writing config at %s" % path_config)
		write_blank(path_config)
	if not os.path.isfile(path_db):
		logging.debug("Writing empty db at: %s" % path_db)
		db = DbManager(path_db)
		db.init()

def clear_userdata():
	path_config = get_conf_path()
	path_db = get_db_path()
	conf_files = [path_config, path_db]
	for conf_file in conf_files:
		if os.path.isfile(conf_file):
			os.unlink(conf_file)

def get_db_path():
	return os.path.join(ad.user_data_dir, DB_NAME)

def get_conf_path():
	return os.path.join(ad.user_data_dir, CNF_NAME)

def defaults_write(parser):
	for default_section in CONFIG_DEFAULTS.keys():
		if not parser.has_section(default_section):
			parser.add_section(default_section)
		for key in CONFIG_DEFAULTS[default_section]:
			if not parser.has_option(default_section, key):
				parser.set(default_section, key, CONFIG_DEFAULTS[default_section][key])
	return parser

def defaults_check(parser):
	for section in CONFIG_REQUIRED.keys():
		if not parser.has_section(section):
			raise BadConfig("Require %s section in config" % section)
			parser.add_section(section)
		for i in CONFIG_REQUIRED[section]:
			if not parser.has_option(section, i):
				raise BadConfig("Require %s option in %s section of config" % (i, section))
			if len(parser.get(section, i)) < 1:
				raise BadConfig("Require non-empty %s option in %s section of config" % (i, section))
	return parser

def test_auth():
	ca = CexAPI(get('cex.username'), get('cex.apikey'), get('cex.secret'))
	t = ca.req('balance')
	if t and 'timestamp' in t:
		print "Works!"
	else:
		logging.error("Check your configuration settings.")

def write_blank(file_path):
	"""Writes a blank config file at path provided by appdir"""
	global _parser
	_parser = defaults_write(_parser)
	write_config()

def write_config():
	path_config = get_conf_path()
	with open(path_config, 'wb') as cf:
		_parser.write(cf)
	logging.info("Config file written to %s" % path_config)

def edit_config():
	"""Edit config file in path from appdir"""
	path_config = get_conf_path()
	editor = os.environ.get('EDITOR','vim')
	subprocess.call([editor, path_config])
	return True

def get_config():
	_parser.read(get_conf_path())
	try:
		defaults_check(_parser)
	except BadConfig, e:
		logging.error(str(e))
	return _parser

def list():
	parser = get_config()
	for section in parser.sections():
		for (n, v) in parser.items(section):
			cprint("%s.%s" % (section, n))
	return True

def parse_name(name):
	try:
		section, cname = name.split('.', 1)
		assert section
		assert cname
	except Exception:
		logging.error("Invalid config option: %s" % name)
		return False
	parser = get_config()
	if not parser.has_section(section):
		logging.error("No such config section: %s" % section)
		return False
	if not parser.has_option(section, cname):
		logging.error("No such config option: %s" % name)
		return False
	return section, cname

def cprint(name):
	section, cname = parse_name(name)
	cval = get(name)
	print "%s.%s=%s" % (section, cname, cval)

def get(name):
	"""get a config option. format = section.name"""
	section, cname = parse_name(name)
	parser = get_config()
	return parser.get(section, cname)

def set(name, value):
	"""set a config option. format = section.name"""
	try:
		section, cname = name.split('.', 1)
		assert section
		assert cname
	except Exception:
		logging.error("Invalid config option: %s" % name)
		return False
	parser = get_config()
	if not parser.has_section(section):
		logging.error("No such config section: %s" % section)
		return False
	if not parser.has_option(section, cname):
		logging.error("No such config option: %s" % name)
		return False
	parser.set(section, cname, value)
	write_config()
	return True



#!/usr/bin/env python
"""
	cexbot.updater - config abstraction
"""

import requests
import cexbot
import logging
# @TODO find a non-brainded semver implementation
import semantic_version

UPDATE_URL = "https://raw.github.com/nikcub/cexbot/master/VERSION"

def get_latest():
	v =requests.get(UPDATE_URL)
	if v.status_code == 200:
		return v.text
	else:
		return False

def check_update():
	cur_version = cexbot.get_version(semantic=True)
	latest_version = get_latest()
	print "Current version: %s Latest version: %s" % (cur_version, latest_version)
	if latest_version:
		lv = semantic_version.Version(latest_version)
		cv = semantic_version.Version(cur_version)
		if lv > cv:
			logging.error("New version available, run git pull or pip -U cexbot")
	return None


#!/usr/bin/env python

import sys
import os
import platform
import cexbot

try:
  from setuptools import setup
except ImportError:
  from distutils.core import setup

if getattr(sys, 'version_info', (0, 0, 0)) < (2, 5, 0, 'final'):
    raise SystemExit("floyd requires Python 2.5 or later.")

if sys.argv[-1] == 'publish':
  os.system('python setup.py sdist upload')
  sys.exit()

scripts = ['bin/cexbot']
packages = [
  'requests',
  'semantic_version'
]

package_dir = os.path.realpath(os.path.dirname(__file__))

def get_file_contents(file_path):
  """Get the context of the file using full path name"""
  full_path = os.path.join(package_dir, file_path)
  return open(full_path, 'r').read()

setup(
  name = 'cexbot',
  description = cexbot.__doc__.split('\n\n')[0],
  long_description = get_file_contents('README.md'),
  keywords = 'cexbot, bitcoin, finance',
  url = 'https://github.com/nikcub/cexbot',
  platforms = ['linux', 'osx'],
  version = cexbot.get_version(),
  author = 'Nik Cubrilovic',
  author_email = 'nikcub@gmail.com',
  license = get_file_contents('LICENSE'),
  install_requires = packages,
  packages = ['cexbot'],
  # scripts = scripts,
  entry_points={
    'console_scripts': [
      "cexbot-cli = cexbot.command_utils:run_cl"
    ],
    # 'gui_scripts': [
      # "cexbot-gui = cexbot.command_utils:run_gui"
    # ]
  },
)


#!/usr/bin/env python

"""cexbot.gui
"""

import sys

from PyQt4 import QtCore, QtGui

from qtui.main import *

def main():
	app = QtGui.QApplication(sys.argv)
	window = QtGui.QDialog()
	f = Ui_formMain()
	f.setupUi(window)
	window.show()
	sys.exit(app.exec_())

if __name__ == '__main__':
	main()


#!/usr/bin/env python

import sys
import os
import errno
import tempfile
from multiprocessing import Process


class singleinstance:
  """
  Implements a single instance application by creating a lock file based on the full path to the script file.

  This is based upon the singleton class from tendo https://github.com/pycontribs/tendo
  which is under the Python Software Foundation License version 2
  """
  def __init__(self, flavor_id=""):
    import sys
    self.initialized = False
    basename = os.path.splitext(os.path.abspath(sys.argv[0]))[0].replace("/", "-").replace(":", "").replace("\\", "-") + '-%s' % flavor_id + '.lock'
    self.lockfile = os.path.normpath(tempfile.gettempdir() + '/' + basename)

    if sys.platform == 'win32':
      try:
        # file already exists, we try to remove (in case previous execution was interrupted)
        if os.path.exists(self.lockfile):
          os.unlink(self.lockfile)
        self.fd = os.open(self.lockfile, os.O_CREAT | os.O_EXCL | os.O_RDWR)
      except OSError:
        type, e, tb = sys.exc_info()
        if e.errno == 13:
          print 'Another instance of this application is already running'
          sys.exit(-1)
        print(e.errno)
        raise
    else:  # non Windows
      import fcntl  # @UnresolvedImport
      self.fp = open(self.lockfile, 'w')
      try:
        fcntl.lockf(self.fp, fcntl.LOCK_EX | fcntl.LOCK_NB)
      except IOError:
        print 'Another instance of this application is already running'
        sys.exit(-1)
    self.initialized = True

  def __del__(self):
    import sys
    if not self.initialized:
      return
    try:
      if sys.platform == 'win32':
        if hasattr(self, 'fd'):
          os.close(self.fd)
          os.unlink(self.lockfile)
      else:
        import fcntl  # @UnresolvedImport
        fcntl.lockf(self.fp, fcntl.LOCK_UN)
        if os.path.isfile(self.lockfile):
          os.unlink(self.lockfile)
    except Exception, e:
      sys.exit(-1)
